/**
 * Copyright (C) 2013 Matteo Tamigi
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */


//DEBUG
#include <iostream>
#include <time.h>
using namespace std;

#include "pdfworkspace.hpp"


PDFWorkspace :: PDFWorkspace () {
     crack = NULL;
     stato = new int[32];
     lenStato = 0;
     max = 0;
     found = false;
}

PDFWorkspace :: PDFWorkspace ( EncWorkSpace enc , unsigned _min = 1 , unsigned _max = 5 ,
                               string chs = STD_CHARSET , bool tp = USER) {
    //Costruttore: istanzio le variabili necessarie e imposto l'ambiente
    //adatto al cracking (provvedo al numero di core da usare, gli algoritmi, ecc.)
    crack = NULL;
    stato = new int[32];
    lenStato = 0;
    lenCs = chs.size();
    found = false;
    SetWs(enc,_min,_max,chs,tp);
}

PDFWorkspace :: ~PDFWorkspace () {
    if( crack )
        delete crack;
    delete [] stato;
}

void PDFWorkspace :: SetWs ( EncWorkSpace enc , unsigned _min = 1 ,
                             unsigned _max = 5 , string chs = STD_CHARSET, bool tp = USER ) {
    //Copio l'impostazione inserita e predispondo il sistema per partire
    //-Genero le classi con i parametri di cifratura corretti
    encWs = enc;
    //Eseguo i settaggi e le allocazioni base
    if( enc.V == 1 && enc.R == 2  )
        crack = new PDFCrackV1R2;
    else if ( enc.V == 2 && enc.R == 3  )
        crack = new PDFCrackV2R3;
    //Carico le informazioni iniziali per ogni thread (comuni a tutte)
    crack->SetWs(enc);
    crack->SetCharset(chs);
    crack->SetTypeCracking(tp);
    //Genero il vettore di stato
    for(unsigned i = 0; i < 32; i++)
        if( i < _min )
            stato[i] = 0;
        else
            stato[i] = -1;
    lenStato = _min;
    max = _max;
    min = _min;
    lenCs = chs.size();
}

void PDFWorkspace :: SetNextStatus () {
    //Determino la combinazione successiva
    if ( ++stato[lenStato] >= lenCs ) {
        stato[lenStato++] = 0;
        if( lenStato < max )
            stato[lenStato] = 0;
    }
    //Se è corretta azzero, altrimenti blocco tutto
    if(lenStato == max)
        ciclo = true;       //Ho finito le combinazioni, termino il ciclo
}

void PDFWorkspace :: run () {
    ciclo = false;
    found = false;
    while ( !ciclo ) {
        crack->SetCiphers(lenStato);
        crack->SetState(lenStato,stato);
        crack->run();
        //Genero un nuovo stato
        SetNextStatus();
        if( crack->GetFoundUserPswd() ) {
            pswd = crack->GetUserPassword();
            found = true;
            ciclo = true;
        }
        if( crack->GetFoundOwnerPswd() ) {
            pswd = crack->GetOwnerPassword();
            found = true;
            ciclo = true;
        }
        
    }
}
